//+------------------------------------------------------------------+
//|                                 UltraSpearmanRankCorrelation.mq5 |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
//|     SmoothAlgorithms.mqh                  |
//|    : __\\MQL5\Include |
//+------------------------------------------------------------------+
//----  
#property copyright "Copyright  2011, Nikolay Kositsin"
//----    
#property link "farria@mail.redcom.ru"
//----   
#property version   "1.00"
//----     
#property indicator_separate_window
//----       3 
#property indicator_buffers 3
//----  1  
#property indicator_plots   1
//+-----------------------------------+ 
//|                 |
//+-----------------------------------+ 
#define RESET 0 //        
//+-----------------------------------+
//|          |
//+-----------------------------------+
//----        
#property indicator_type1   DRAW_COLOR_HISTOGRAM2
//----      
#property indicator_color1  Gray,Magenta,HotPink,Red,Brown,RoyalBlue,MediumBlue,DeepSkyBlue,LightSkyBlue
//----     4
#property indicator_width1 4
//----   
#property indicator_label1 "Ultra SpearmanRankCorrelation"
//+-----------------------------------+
//|    CXMA             |
//+-----------------------------------+
#include <SmoothAlgorithms.mqh> 
//+-----------------------------------+
//----    CXMA   SmoothAlgorithms.mqh
CXMA XMA[];
//+-----------------------------------+
//|             |
//+-----------------------------------+  
enum WIDTH
  {
   Width_1=1, // 1
   Width_2,   // 2
   Width_3,   // 3
   Width_4,   // 4
   Width_5    // 5
  };
//+-----------------------------------+
//|             |
//+-----------------------------------+
enum STYLE
  {
   SOLID_,       //  
   DASH_,        //  
   DOT_,         //  
   DASHDOT_,     // - 
   DASHDOTDOT_   // -    
  };
//+----------------------------------------------+
//|                    |
//+----------------------------------------------+
input int  rangeN=14;
//----
input Smooth_Method W_Method=MODE_JJMA;             //  
input int StartLength=3;                            //                       
input int WPhase=100;                               //  
//----  
input uint Step=5;                                  //   
input uint StepsTotal=10;                           //   
//----
input Smooth_Method SmoothMethod=MODE_JJMA;         //  
input int SmoothLength=3;                           //  
input int SmoothPhase=100;                          //  
//----                          
input uint UpLevel=80;                              //    %%
input uint DnLevel=20;                              //    %%
input color UpLevelsColor=Blue;                     //   
input color DnLevelsColor=Blue;                     //   
input STYLE Levelstyle=DASH_;                       //  
input WIDTH  LevelsWidth=Width_1;                   //                         
//+----------------------------------------------+
//----   ,   
//----      
double BullsBuffer[];
double BearsBuffer[];
double ColorBuffer[];
//----          SpearmanRankCorrelation
int period[];
//----         SpearmanRankCorrelation
double invalue0[],invalue1[];
//----
double dUpLevel,dDnLevel;
//----      
int SRC_Handle;
//----      
int Maxrange=30;
uint StTot1,StTot2;
int min_rates_total,min_rates_src,min_rates_xma;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+  
int OnInit()
  {
//----     
   int size=int(StepsTotal+3);
   if(ArrayResize(XMA,size)<size) Print("      XMA[]");
   size-=2;
   if(ArrayResize(invalue0,size)<size) Print("      invalue0[]");
   if(ArrayResize(invalue1,size)<size) Print("      invalue1[]");
   if(ArrayResize(period,size)<size) Print("      period[]");

   ArrayInitialize(invalue0,0);
   ArrayInitialize(invalue1,0);
   ArrayInitialize(period,0);   
   
//----    SpearmanRankCorrelation
   SRC_Handle=iCustom(NULL,0,"SpearmanRankCorrelation",rangeN,0,Maxrange,true);
   if(SRC_Handle==INVALID_HANDLE)
     {
      Print("      SpearmanRankCorrelation");
      return(1);
     }

//----  
   for(int sm=int(StepsTotal); sm>=0 && !IsStopped(); sm--) period[sm]=int(StartLength+sm*Step);
   dUpLevel=StepsTotal*UpLevel/100.0;
   dDnLevel=StepsTotal*DnLevel/100.0;

//----       
   XMA[0].XMALengthCheck("StartLength", StartLength);
   XMA[0].XMAPhaseCheck("WPhase", WPhase, W_Method);
   XMA[0].XMALengthCheck("SmoothLength", SmoothLength);
   XMA[0].XMAPhaseCheck("SmoothPhase", SmoothPhase, SmoothMethod);

//----     
   min_rates_src=rangeN;
   min_rates_xma=min_rates_src+XMA[0].GetStartBars(W_Method,StartLength+Step*StepsTotal,WPhase)+1;
   min_rates_total=min_rates_xma+XMA[0].GetStartBars(SmoothMethod,SmoothLength,SmoothPhase);
   StTot1=StepsTotal+1;
   StTot2=StepsTotal+2;

//----    BullsBuffer[]   
   SetIndexBuffer(0,BullsBuffer,INDICATOR_DATA);
//----       1  min_rates_total
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,min_rates_total);
//----       
   ArraySetAsSeries(BullsBuffer,true);

//----    BearsBuffer[]   
   SetIndexBuffer(1,BearsBuffer,INDICATOR_DATA);
//----       2  min_rates_total
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,min_rates_total);
//----       
   ArraySetAsSeries(BearsBuffer,true);

//----     ,     
   SetIndexBuffer(2,ColorBuffer,INDICATOR_COLOR_INDEX);
//----      
   PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,min_rates_total+1);
//----       
   ArraySetAsSeries(ColorBuffer,true);

//----      
   string shortname="Ultra Spearman RankCorrelation";
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,2);

//----     
   IndicatorSetInteger(INDICATOR_LEVELS,2);

   IndicatorSetDouble(INDICATOR_LEVELVALUE,0,dUpLevel);
   IndicatorSetInteger(INDICATOR_LEVELCOLOR,0,UpLevelsColor);
   IndicatorSetInteger(INDICATOR_LEVELSTYLE,0,Levelstyle);
   IndicatorSetInteger(INDICATOR_LEVELWIDTH,0,LevelsWidth);

   IndicatorSetDouble(INDICATOR_LEVELVALUE,1,dDnLevel);
   IndicatorSetInteger(INDICATOR_LEVELCOLOR,1,DnLevelsColor);
   IndicatorSetInteger(INDICATOR_LEVELSTYLE,1,Levelstyle);
   IndicatorSetInteger(INDICATOR_LEVELWIDTH,1,LevelsWidth);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double& high[],     //       
                const double& low[],      //        
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//----       
   if(BarsCalculated(SRC_Handle)<rates_total
      || rates_total<min_rates_total)
      return(RESET);

//----    
   int to_copy,limit,bar,maxbar0,maxbar1,maxbar2;
   double upsch,dnsch,SRC[];

//----    maxbar   XMASeries()
   maxbar0=rates_total-1;
   maxbar1=maxbar0-min_rates_src;
   maxbar2=maxbar0-min_rates_xma;

//----    limit    
   if(prev_calculated>rates_total || prev_calculated<=0)//      
      limit=maxbar0;                       //      
   else limit=rates_total-prev_calculated; //      
   to_copy=limit+1;

//----         
   ArraySetAsSeries(open,true);
   ArraySetAsSeries(low,true);
   ArraySetAsSeries(high,true);
   ArraySetAsSeries(close,true);
   ArraySetAsSeries(SRC,true);
   
//----      
   if(CopyBuffer(SRC_Handle,0,0,to_copy,SRC)<=0) return(RESET);

//----    
   for(bar=limit; bar>=0 && !IsStopped(); bar--)
     {
      for(int sm=int(StepsTotal); sm>=0 && !IsStopped(); sm--)
         invalue0[sm]=XMA[sm].XMASeries(maxbar1,prev_calculated,rates_total,W_Method,WPhase,period[sm],SRC[bar],bar,true);

      if(bar>maxbar2)
        {
         if(bar) ArrayCopy(invalue1,invalue0,0,0,WHOLE_ARRAY);
         continue;
        }

      upsch=0;
      dnsch=0;
      for(int sm=int(StepsTotal); sm>=0 && !IsStopped(); sm--)
         if(invalue0[sm]>invalue1[sm]) upsch++;
      else                          dnsch++;

      BullsBuffer[bar]=XMA[StTot1].XMASeries(maxbar2,prev_calculated,rates_total,SmoothMethod,SmoothPhase,SmoothLength,upsch,bar,true);
      BearsBuffer[bar]=XMA[StTot2].XMASeries(maxbar2,prev_calculated,rates_total,SmoothMethod,SmoothPhase,SmoothLength,dnsch,bar,true);

      if(bar) ArrayCopy(invalue1,invalue0,0,0,WHOLE_ARRAY);
     }

//----    limit    
   if(prev_calculated>rates_total || prev_calculated<=0)//      
      limit--;

//----    
//----    
   for(bar=limit; bar>=0 && !IsStopped(); bar--)
     {
      ColorBuffer[bar]=0;

      if(BullsBuffer[bar]>BearsBuffer[bar])
        {
         if(BullsBuffer[bar]>dUpLevel || BearsBuffer[bar]<dDnLevel)
           {
            if(BullsBuffer[bar+1]<=BullsBuffer[bar]) ColorBuffer[bar]=7;
            else ColorBuffer[bar]=8;
           }
         else
           {
            if(BullsBuffer[bar+1]<=BullsBuffer[bar]) ColorBuffer[bar]=5;
            else ColorBuffer[bar]=6;
           }
        }

      if(BullsBuffer[bar]<BearsBuffer[bar])
        {
         if(BullsBuffer[bar]<dDnLevel || BearsBuffer[bar]>dUpLevel)
           {
            if(BearsBuffer[bar+1]<=BearsBuffer[bar]) ColorBuffer[bar]=1;
            else ColorBuffer[bar]=2;
           }
         else
           {
            if(BearsBuffer[bar+1]<=BearsBuffer[bar]) ColorBuffer[bar]=3;
            else ColorBuffer[bar]=4;
           }
        }
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
